import {
    Switch,
    Route,
    Link,
    useHistory
  } from 'react-router-dom';

import {useEffect, useState} from 'react';

import Signin from './Signin'
import Signup from './Signup'
import Home from './Home'
import useLocalState from './localState';

let Menu = () => {
    let history = useHistory();
    let [token, setToken] = useLocalState('token', null);
    let [user, setUser] = useState(null);
    let [userCheck, setUserCheck] = useState(false);

    let checkStatus = res => {
        if (res.ok) {
            return res;
        } else {
            return res.text()
            .then(msg => { throw new Error(msg); });
        }
    };

    let signin = (user, password) => {
        return fetch(`http://127.0.0.1:4200/signin`, {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ 'username': user, 'password': password })
        })
        .then(checkStatus)
        .then(res => res.json())
        .then(data => {
            setUser(data.user);
            setToken(data.token);
            history.push('/');
        });
    };

    let signup = (user, password) => {
        return fetch(`http://127.0.0.1:4200/signup`, {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({ 'username': user, 'password': password })
        })
        .then(checkStatus)
        .then(res => res.json())
        .then(() => {
            history.push('/signin');
        });
    };

    let signout = () => {
        setUser(null);
        setToken(null);
    }

    let checkUser = () => {
        setUserCheck(true);
        if(token) {
            fetch(`http://127.0.0.1:4200/whoami`, {
                method: 'GET',
                headers: { 
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + token
                }
            })
            .then(checkStatus)
            .then(res => res.json())
            .then(data => {
                setUser(data);
                console.log("Connecté");
            })
            .catch(err => { console.log(err) });
        }
        setUserCheck(false);
    }

    useEffect(() => {
        checkUser();
    }, []);

    return (
        <div>
            { userCheck ? 'Checking for Authentification...' :
                <div>
                    <nav>
                        <ul>
                            <li>
                                <Link to='/'>Home</Link>
                            </li>
                            {user ?
                                <li>
                                    Connected as {user.username} <button onClick={signout}> Signout</button>
                                </li> : 
                                <div>
                                    <li>
                                        <Link to='/Signin'>Signin</Link>
                                    </li>
                                    <li>
                                        <Link to='/Signup'>Signup</Link>
                                    </li>
                                </div>
                            }
                        </ul>
                    </nav>
                    <Switch>
                        <Route path='/signin'>
                            <Signin signin={signin}/>
                        </Route>
                        <Route path='/signup'>
                            <Signup signup={signup}/>
                        </Route>
                        <Route path='/'>
                            <Home />
                        </Route>
                    </Switch>
                </div>
            }
        </div>
    );
};

export default Menu;
